@@ -103,9 +103,9 @@ void wtokensplit(char *command, char ***argv, int *argc)
 		token = wtokennext(line, &line);
 		if (token) {
 			if (count == 0)
-				*argv = wmalloc(sizeof(char **));
+				*argv = wmalloc(sizeof(**argv));
 			else
-				*argv = wrealloc(*argv, (count + 1) * sizeof(char **));
+				*argv = wrealloc(*argv, (count + 1) * sizeof(**argv));
 			(*argv)[count++] = token;
 		}
 	} while (token != NULL && line != NULL);
@@ -204,7 +204,7 @@ char *wstrndup(const char *str, size_t len)
 char *wstrconcat(const char *str1, const char *str2)
 {
 	char *str;
-	size_t slen;
+	size_t slen, slen1;
 
 	if (!str1 && str2)
 		return wstrdup(str2);
@@ -213,13 +213,11 @@ char *wstrconcat(const char *str1, const char *str2)
 	else if (!str1 && !str2)
 		return NULL;
 
-	slen = strlen(str1) + strlen(str2) + 1;
+	slen1 = strlen(str1);
+	slen = slen1 + strlen(str2) + 1;
 	str = wmalloc(slen);
-	if (wstrlcpy(str, str1, slen) >= slen ||
-	    wstrlcat(str, str2, slen) >= slen) {
-		wfree(str);
-		return NULL;
-	}
+	strcpy(str, str1);
+	strcpy(str + slen1, str2);
 
 	return str;
 }
@@ -235,14 +233,13 @@ char *wstrappend(char *dst, const char *src)
 
 	slen = strlen(dst) + strlen(src) + 1;
 	dst = wrealloc(dst, slen);
-	if (wstrlcat(dst, src, slen) >= slen)
-		return NULL;
+	strcat(dst, src);
 
 	return dst;
 }
 
 
-#if HAVE_STRLCAT
+#ifdef HAVE_STRLCAT
 size_t
 wstrlcat(char *dst, const char *src, size_t siz)
 {
@@ -303,7 +300,7 @@ wstrlcat(char *dst, const char *src, size_t siz)
 }
 #endif /* HAVE_STRLCAT */
 
-#if HAVE_STRLCPY
+#ifdef HAVE_STRLCPY
 size_t
 wstrlcpy(char *dst, const char *src, size_t siz)
 {
