@@ -35,8 +35,6 @@
 #endif
 
 char *NOptionValueChanged = "NOptionValueChanged";
-Bool xext_xkb_supported = False;
-
 
 #define MAX_DEATHS	64
 
@@ -157,8 +155,6 @@ int main(int argc, char **argv)
 		exit(0);
 	}
 
-	xext_xkb_supported = XkbQueryExtension(dpy, NULL, NULL, NULL, NULL, NULL);
-
 	WMPLSetCaseSensitive(False);
 
 	Initialize(scr);
