@@ -45,20 +45,23 @@
 
 #include "WindowMaker.h"
 #include "window.h"
-#include "funcs.h"
+#include "defaults.h"
+#include "event.h"
+#include "startup.h"
 #include "menu.h"
 #include "keybind.h"
 #include "xmodifier.h"
 #include "session.h"
+#include "shutdown.h"
 #include "dialog.h"
+#include "main.h"
+#include "monitor.h"
+#include "misc.h"
 
 #include <WINGs/WUtil.h>
 
-#ifndef GLOBAL_DEFAULTS_SUBDIR
-#define GLOBAL_DEFAULTS_SUBDIR "WindowMaker"
-#endif
-
 /****** Global Variables ******/
+struct wmaker_global_variables w_global;
 
 /* general info */
 
@@ -66,104 +69,25 @@ Display *dpy;
 
 char *ProgName;
 
-unsigned int ValidModMask = 0xff;
-
-#ifdef HAVE_INOTIFY
-int inotifyFD;
-int inotifyWD;
-#endif
-/* locale to use. NULL==POSIX or C */
-char *Locale = NULL;
-
-int wScreenCount = 0;
-
-WPreferences wPreferences;
+struct WPreferences wPreferences;
 
 WShortKey wKeyBindings[WKBD_LAST];
 
-/* defaults domains */
-WDDomain *WDWindowMaker = NULL;
-WDDomain *WDWindowAttributes = NULL;
-WDDomain *WDRootMenu = NULL;
-
-/* XContexts */
-XContext wWinContext;
-XContext wAppWinContext;
-XContext wStackContext;
-XContext wVEdgeContext;
-
-/* Atoms */
-Atom _XA_WM_STATE;
-Atom _XA_WM_CHANGE_STATE;
-Atom _XA_WM_PROTOCOLS;
-Atom _XA_WM_TAKE_FOCUS;
-Atom _XA_WM_DELETE_WINDOW;
-Atom _XA_WM_SAVE_YOURSELF;
-Atom _XA_WM_CLIENT_LEADER;
-Atom _XA_WM_COLORMAP_WINDOWS;
-Atom _XA_WM_COLORMAP_NOTIFY;
-
-Atom _XA_GNUSTEP_WM_ATTR;
-Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
-Atom _XA_GNUSTEP_WM_RESIZEBAR;
-Atom _XA_GNUSTEP_TITLEBAR_STATE;
-
-Atom _XA_WINDOWMAKER_MENU;
-Atom _XA_WINDOWMAKER_WM_PROTOCOLS;
-Atom _XA_WINDOWMAKER_STATE;
-
-Atom _XA_WINDOWMAKER_WM_FUNCTION;
-Atom _XA_WINDOWMAKER_NOTICEBOARD;
-Atom _XA_WINDOWMAKER_COMMAND;
-
-Atom _XA_WINDOWMAKER_ICON_SIZE;
-Atom _XA_WINDOWMAKER_ICON_TILE;
-
-Atom _XA_WM_IGNORE_FOCUS_EVENTS;
-
-/* cursors */
-Cursor wCursor[WCUR_LAST];
-
-/* last event timestamp for XSetInputFocus */
-Time LastTimestamp = CurrentTime;
-/* timestamp on the last time we did XSetInputFocus() */
-Time LastFocusChange = CurrentTime;
-
-#ifdef SHAPE
-Bool wShapeSupported;
-int wShapeEventBase;
-#endif
-
-#ifdef KEEP_XKB_LOCK_STATUS
-Bool wXkbSupported;
-int wXkbEventBase;
-#endif
-
-#ifdef HAVE_XRANDR
-Bool has_randr;
-int randr_event_base;
-#endif
-
-/* special flags */
-char WProgramSigState = 0;
-char WProgramState = WSTATE_NORMAL;
-char WDelayedActionSet = 0;
-
 /* notifications */
-const char *WMNManaged = "WMNManaged";
-const char *WMNUnmanaged = "WMNUnmanaged";
-const char *WMNChangedWorkspace = "WMNChangedWorkspace";
-const char *WMNChangedState = "WMNChangedState";
-const char *WMNChangedFocus = "WMNChangedFocus";
-const char *WMNChangedStacking = "WMNChangedStacking";
-const char *WMNChangedName = "WMNChangedName";
+const char WMNManaged[] = "WMNManaged";
+const char WMNUnmanaged[] = "WMNUnmanaged";
+const char WMNChangedWorkspace[] = "WMNChangedWorkspace";
+const char WMNChangedState[] = "WMNChangedState";
+const char WMNChangedFocus[] = "WMNChangedFocus";
+const char WMNChangedStacking[] = "WMNChangedStacking";
+const char WMNChangedName[] = "WMNChangedName";
 
-const char *WMNWorkspaceCreated = "WMNWorkspaceCreated";
-const char *WMNWorkspaceDestroyed = "WMNWorkspaceDestroyed";
-const char *WMNWorkspaceChanged = "WMNWorkspaceChanged";
-const char *WMNWorkspaceNameChanged = "WMNWorkspaceNameChanged";
+const char WMNWorkspaceCreated[] = "WMNWorkspaceCreated";
+const char WMNWorkspaceDestroyed[] = "WMNWorkspaceDestroyed";
+const char WMNWorkspaceChanged[] = "WMNWorkspaceChanged";
+const char WMNWorkspaceNameChanged[] = "WMNWorkspaceNameChanged";
 
-const char *WMNResetStacking = "WMNResetStacking";
+const char WMNResetStacking[] = "WMNResetStacking";
 
 /******** End Global Variables *****/
 
@@ -173,10 +97,6 @@ static char **Arguments;
 
 static int ArgCount;
 
-extern void EventLoop(void);
-extern void StartUp(Bool defaultScreenOnly);
-extern int MonitorLoop(int argc, char **argv);
-
 static Bool multiHead = True;
 
 static int *wVisualID = NULL;
@@ -215,7 +135,7 @@ static void setWVisualID(int screen, int val)
 		int oldlen = wVisualID_len;
 
 		wVisualID_len = screen + 1;
-		wVisualID = (int *)realloc(wVisualID, wVisualID_len * sizeof(int));
+		wVisualID = (int *)wrealloc(wVisualID, wVisualID_len * sizeof(int));
 		for (i = oldlen; i < wVisualID_len; i++) {
 			wVisualID[i] = -1;
 		}
@@ -273,11 +193,14 @@ static int initWVisualID(const char *user_str)
 	return 0;
 }
 
-void Exit(int status)
+noreturn void Exit(int status)
 {
 	if (dpy)
 		XCloseDisplay(dpy);
 
+	RShutdown(); /* wrlib clean exit */
+	wutil_shutdown();  /* WUtil clean-up */
+
 	exit(status);
 }
 
@@ -320,9 +243,22 @@ void SetupEnvironment(WScreen * scr)
 		int len = strlen(DisplayName) + 64;
 		tmp = wmalloc(len);
 		snprintf(tmp, len, "DISPLAY=%s", XDisplayName(DisplayName));
-		ptr = strchr(strchr(tmp, ':'), '.');
-		if (ptr)
-			*ptr = 0;
+
+		/* Search from the end to be compatible with ipv6 address */
+		ptr = strrchr(tmp, ':');
+		if (ptr == NULL) {
+			static Bool message_already_displayed = False;
+
+			if (!message_already_displayed)
+				wwarning(_("the display name has an unexpected syntax: \"%s\""),
+				         XDisplayName(DisplayName));
+			message_already_displayed = True;
+		} else {
+			/* If found, remove the screen specification from the display variable */
+			ptr = strchr(ptr, '.');
+			if (ptr)
+				*ptr = 0;
+		}
 		snprintf(buf, sizeof(buf), ".%i", scr->screen);
 		strcat(tmp, buf);
 		putenv(tmp);
@@ -338,8 +274,13 @@ typedef struct {
 	char *command;
 } _tuple;
 
-static void shellCommandHandler(pid_t pid, unsigned char status, _tuple * data)
+static void shellCommandHandler(pid_t pid, unsigned int status, void *client_data)
 {
+	_tuple *data = (_tuple *) client_data;
+
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) pid;
+
 	if (status == 127) {
 		char *buffer;
 
@@ -357,7 +298,7 @@ static void shellCommandHandler(pid_t pid, unsigned char status, _tuple * data)
 	wfree(data);
 }
 
-void ExecuteShellCommand(WScreen * scr, char *command)
+void ExecuteShellCommand(WScreen *scr, const char *command)
 {
 	static char *shell = NULL;
 	pid_t pid;
@@ -394,7 +335,7 @@ void ExecuteShellCommand(WScreen * scr, char *command)
 		data->scr = scr;
 		data->command = wstrdup(command);
 
-		wAddDeathHandler(pid, (WDeathHandler *) shellCommandHandler, data);
+		wAddDeathHandler(pid, shellCommandHandler, data);
 	}
 }
 
@@ -414,11 +355,19 @@ Bool RelaunchWindow(WWindow *wwin)
 
 	char **argv;
 	int argc;
+	char *command = NULL;
 
-	if (! XGetCommand(dpy, wwin->client_win, &argv, &argc) || argc == 0 || argv == NULL) {
+	command = GetCommandForWindow(wwin->client_win);
+	if (!command) {
+#ifdef USE_XRES
+		werror("cannot relaunch the application because no associated process found");
+#else
 		werror("cannot relaunch the application because no WM_COMMAND property is set");
+#endif
 		return False;
-	}
+	} else
+		wtokensplit(command, &argv, &argc);
+
 
 	pid_t pid = fork();
 
@@ -444,18 +393,19 @@ Bool RelaunchWindow(WWindow *wwin)
 	} else if (pid < 0) {
 		werror("cannot fork a new process");
 
-		XFreeStringList(argv);
+		wtokenfree(argv, argc);
+		wfree(command);
 		return False;
 	} else {
 		_tuple *data = wmalloc(sizeof(_tuple));
 
 		data->scr = wwin->screen_ptr;
-		data->command = wtokenjoin(argv, argc);
+		data->command = command;
 
 		/* not actually a shell command */
-		wAddDeathHandler(pid, (WDeathHandler *) shellCommandHandler, data);
+		wAddDeathHandler(pid, shellCommandHandler, data);
 
-		XFreeStringList(argv);
+		wtokenfree(argv, argc);
 	}
 
 	return True;
@@ -468,12 +418,12 @@ Bool RelaunchWindow(WWindow *wwin)
  *
  *----------------------------------------------------------------------
  */
-void wAbort(Bool dumpCore)
+noreturn void wAbort(Bool dumpCore)
 {
 	int i;
 	WScreen *scr;
 
-	for (i = 0; i < wScreenCount; i++) {
+	for (i = 0; i < w_global.screen_count; i++) {
 		scr = wScreenWithNumber(i);
 		if (scr)
 			RestoreDesktop(scr);
@@ -485,33 +435,34 @@ void wAbort(Bool dumpCore)
 		exit(1);
 }
 
-void print_help()
+static void print_help(void)
 {
 	printf(_("Usage: %s [options]\n"), ProgName);
 	puts(_("The Window Maker window manager for the X window system"));
 	puts("");
 	puts(_(" -display host:dpy	display to use"));
-#ifdef USECPP
-	puts(_(" --no-cpp 		disable preprocessing of configuration files"));
+#ifdef USE_ICCCM_WMREPLACE
+	puts(_(" --replace		replace running window manager"));
 #endif
 	puts(_(" --no-dock		do not open the application Dock"));
 	puts(_(" --no-clip		do not open the workspace Clip"));
 	puts(_(" --no-autolaunch	do not autolaunch applications"));
+	puts(_(" --no-drawer		disable drawers in the dock"));
 	puts(_(" --dont-restore		do not restore saved session"));
 
 	puts(_(" --locale locale	locale to use"));
 
-	puts(_(" --create-stdcmap	create the standard colormap hint in PseudoColor visuals"));
 	puts(_(" --visual-id visualid	visual id of visual to use"));
 	puts(_(" --static		do not update or save configurations"));
 #ifndef HAVE_INOTIFY
 	puts(_(" --no-polling		do not periodically check for configuration updates"));
 #endif
+	puts(_(" --global_defaults_path	print the path for default config and exit"));
 	puts(_(" --version		print version and exit"));
 	puts(_(" --help			show this message"));
 }
 
-void check_defaults()
+static void check_defaults(void)
 {
 	char *path;
 
@@ -537,37 +488,39 @@ void check_defaults()
  * files have changed, using linux kernel inotify mechanism
  */
 
-static void inotifyWatchConfig()
+static void inotifyWatchConfig(void)
 {
 	char *watchPath = NULL;
-	inotifyFD = inotify_init();	/* Initialise an inotify instance */
-	if (inotifyFD < 0) {
+
+	w_global.inotify.fd_event_queue = inotify_init();	/* Initialise an inotify instance */
+	if (w_global.inotify.fd_event_queue < 0) {
 		wwarning(_("could not initialise an inotify instance."
 			   " Changes to the defaults database will require"
 			   " a restart to take effect. Check your kernel!"));
 	} else {
-		watchPath = wstrconcat(wusergnusteppath(), "/Defaults");
+		watchPath = wdefaultspathfordomain("");
 		/* Add the watch; really we are only looking for modify events
 		 * but we might want more in the future so check all events for now.
 		 * The individual events are checked for in event.c.
 		 */
-		inotifyWD = inotify_add_watch(inotifyFD, watchPath, IN_ALL_EVENTS);
-		if (inotifyWD < 0) {
+		w_global.inotify.wd_defaults = inotify_add_watch(w_global.inotify.fd_event_queue, watchPath, IN_ALL_EVENTS);
+		if (w_global.inotify.wd_defaults < 0) {
 			wwarning(_("could not add an inotify watch on path %s."
-				   "Changes to the defaults database will require"
+				   " Changes to the defaults database will require"
 				   " a restart to take effect."), watchPath);
-			close(inotifyFD);
+			close(w_global.inotify.fd_event_queue);
+			w_global.inotify.fd_event_queue = -1;
 		}
 	}
 	wfree(watchPath);
 }
 #endif /* HAVE_INOTIFY */
 
-static void execInitScript()
+static void execInitScript(void)
 {
 	char *file, *paths;
 
-	paths = wstrconcat(wusergnusteppath(), "/Library/WindowMaker");
+	paths = wstrconcat(wuserdatapath(), "/" PACKAGE_TARNAME);
 	paths = wstrappend(paths, ":" DEF_CONFIG_PATHS);
 
 	file = wfindfile(paths, DEF_INIT_SCRIPT);
@@ -581,11 +534,11 @@ static void execInitScript()
 	}
 }
 
-void ExecExitScript()
+void ExecExitScript(void)
 {
 	char *file, *paths;
 
-	paths = wstrconcat(wusergnusteppath(), "/Library/WindowMaker");
+	paths = wstrconcat(wuserdatapath(), "/" PACKAGE_TARNAME);
 	paths = wstrappend(paths, ":" DEF_CONFIG_PATHS);
 
 	file = wfindfile(paths, DEF_EXIT_SCRIPT);
@@ -604,10 +557,18 @@ int main(int argc, char **argv)
 	int i_am_the_monitor, i, len;
 	char *str, *alt;
 
+	memset(&w_global, 0, sizeof(w_global));
+	w_global.program.state = WSTATE_NORMAL;
+	w_global.program.signal_state = WSTATE_NORMAL;
+	w_global.timestamp.last_event = CurrentTime;
+	w_global.timestamp.focus_change = CurrentTime;
+	w_global.ignore_workspace_change = False;
+	w_global.shortcut.modifiers_mask = 0xff;
+
 	/* setup common stuff for the monitor and wmaker itself */
 	WMInitializeApplication("WindowMaker", &argc, argv);
 
-	memset(&wPreferences, 0, sizeof(WPreferences));
+	memset(&wPreferences, 0, sizeof(wPreferences));
 
 	wPreferences.fallbackWMs = WMCreateArray(8);
 	alt = getenv("WINDOWMAKER_ALT_WM");
@@ -679,12 +640,9 @@ static int real_main(int argc, char **argv)
 
 	if (argc > 1) {
 		for (i = 1; i < argc; i++) {
-#ifdef USECPP
 			if (strcmp(argv[i], "-nocpp") == 0 || strcmp(argv[i], "--no-cpp") == 0) {
-				wPreferences.flags.nocpp = 1;
-			} else
-#endif
-			if (strcmp(argv[i], "--for-real") == 0) {
+				wwarning(_("option \"%s\" is deprecated, please remove it from your script"), argv[i]);
+			} else if (strcmp(argv[i], "--for-real") == 0) {
 				wPreferences.flags.restarting = 0;
 			} else if (strcmp(argv[i], "--for-real=") == 0) {
 				wPreferences.flags.restarting = 1;
@@ -697,13 +655,20 @@ static int real_main(int argc, char **argv)
 				wPreferences.flags.norestore = 1;
 			} else if (strcmp(argv[i], "-nodock") == 0 || strcmp(argv[i], "--no-dock") == 0) {
 				wPreferences.flags.nodock = 1;
+				wPreferences.flags.nodrawer = 1;
 			} else if (strcmp(argv[i], "-noclip") == 0 || strcmp(argv[i], "--no-clip") == 0) {
 				wPreferences.flags.noclip = 1;
+			} else if (strcmp(argv[i], "-nodrawer") == 0 || strcmp(argv[i], "--no-drawer") == 0) {
+				wPreferences.flags.nodrawer = 1;
+#ifdef USE_ICCCM_WMREPLACE
+			} else if (strcmp(argv[i], "-replace") == 0 || strcmp(argv[i], "--replace") == 0) {
+				wPreferences.flags.replace = 1;
+#endif
 			} else if (strcmp(argv[i], "-version") == 0 || strcmp(argv[i], "--version") == 0) {
 				printf("Window Maker %s\n", VERSION);
 				exit(0);
 			} else if (strcmp(argv[i], "--global_defaults_path") == 0) {
-			  printf("%s/%s\n", SYSCONFDIR, GLOBAL_DEFAULTS_SUBDIR);
+				printf("%s\n", PKGCONFDIR);
 				exit(0);
 			} else if (strcmp(argv[i], "-locale") == 0 || strcmp(argv[i], "--locale") == 0) {
 				i++;
@@ -711,7 +676,7 @@ static int real_main(int argc, char **argv)
 					wwarning(_("too few arguments for %s"), argv[i - 1]);
 					exit(0);
 				}
-				Locale = argv[i];
+				w_global.locale = argv[i];
 			} else if (strcmp(argv[i], "-display") == 0 || strcmp(argv[i], "--display") == 0) {
 				i++;
 				if (i >= argc) {
@@ -729,12 +694,14 @@ static int real_main(int argc, char **argv)
 					wwarning(_("bad value for visualid: \"%s\""), argv[i]);
 					exit(0);
 				}
-			} else if (strcmp(argv[i], "-static") == 0 || strcmp(argv[i], "--static") == 0
-#ifndef HAVE_INOTIFY
-				    || strcmp(argv[i], "--no-polling") == 0
-#endif
-				    ) {
+			} else if (strcmp(argv[i], "-static") == 0 || strcmp(argv[i], "--static") == 0) {
 				wPreferences.flags.noupdates = 1;
+			} else if (strcmp(argv[i], "--no-polling") == 0) {
+#ifndef HAVE_INOTIFY
+				wPreferences.flags.noupdates = 1;
+#else
+				wmessage(_("your version of Window Maker was compiled with INotify support, so \"--no-polling\" has no effect"));
+#endif
 			} else if (strcmp(argv[i], "--help") == 0) {
 				print_help();
 				exit(0);
@@ -751,19 +718,19 @@ static int real_main(int argc, char **argv)
 		check_defaults();
 	}
 
-	if (Locale) {
-		setenv("LANG", Locale, 1);
+	if (w_global.locale) {
+		setenv("LANG", w_global.locale, 1);
 	} else {
-		Locale = getenv("LC_ALL");
-		if (!Locale) {
-			Locale = getenv("LANG");
+		w_global.locale = getenv("LC_ALL");
+		if (!w_global.locale) {
+			w_global.locale = getenv("LANG");
 		}
 	}
 
 	setlocale(LC_ALL, "");
 
-	if (!Locale || strcmp(Locale, "C") == 0 || strcmp(Locale, "POSIX") == 0)
-		Locale = NULL;
+	if (!w_global.locale || strcmp(w_global.locale, "C") == 0 || strcmp(w_global.locale, "POSIX") == 0)
+		w_global.locale = NULL;
 #ifdef I18N
 	if (getenv("NLSPATH")) {
 		bindtextdomain("WindowMaker", getenv("NLSPATH"));
@@ -791,11 +758,11 @@ static int real_main(int argc, char **argv)
 	}
 #endif
 
-	if (Locale) {
+	if (w_global.locale) {
 		char *ptr;
 
-		Locale = wstrdup(Locale);
-		ptr = strchr(Locale, '.');
+		w_global.locale = wstrdup(w_global.locale);
+		ptr = strchr(w_global.locale, '.');
 		if (ptr)
 			*ptr = 0;
 	}
@@ -840,7 +807,7 @@ static int real_main(int argc, char **argv)
 	wXModifierInitialize();
 	StartUp(!multiHead);
 
-	if (wScreenCount == 1)
+	if (w_global.screen_count == 1)
 		multiHead = False;
 
 	execInitScript();
