@@ -70,7 +70,7 @@ static const struct {
 	{ "NoTitlebar", { .no_titlebar = 1 }, N_("Disable titlebar"),
 	  N_("Remove the titlebar of this window.\n"
 	     "To access the window commands menu of a window\n"
-	     "without it's titlebar, press Control+Esc (or the\n"
+	     "without its titlebar, press Control+Esc (or the\n"
 	     "equivalent shortcut, if you changed the default\n"
 	     "settings).") },
 
@@ -156,7 +156,11 @@ static const struct {
 
 	{ "NoMiniaturizable", { .no_miniaturizable = 1 }, N_("Do not let it be minimized"),
 	  N_("Do not let the window of this application be\n"
-	     "minimized.\n") }
+	     "minimized.\n") },
+
+	{ "IgnoreDecorationChanges", { .ignore_decoration_changes = 1 }, N_("Ignore decoration changes"),
+	  N_("Ignore any request from the application to change\n"
+	     "window decorations (like hiding the titlebar).\n") }
 
 #ifdef XKB_BUTTON_HINT
 	,{ "NoLanguageButton", { .no_language_button = 1 }, N_("Disable language button"),
@@ -235,7 +239,7 @@ static InspectorPanel *panelList = NULL;
  * save the user choice to the database, but as we will need to convert that name into a Property
  * List, we use here a Cache of Property Lists, generated only once, which can be reused. It will
  * also save on memory because of the re-use of the same storage space instead of allocating a new
- * one everytime.
+ * one every time.
  */
 static WMPropList *pl_attribute[sizeof(window_attribute) / sizeof(window_attribute[0])] = { [0] = NULL };
 static WMPropList *pl_advoptions[sizeof(advanced_option) / sizeof(advanced_option[0])];
@@ -1249,7 +1253,7 @@ static InspectorPanel *createInspectorForWindow(WWindow *wwin, int xpos, int ypo
 
 	wWindowMap(panel->frame);
 
-	showIconFor(WMWidgetScreen(panel->alwChk), panel, wwin->wm_instance, wwin->wm_class, UPDATE_TEXT_FIELD);
+	showIconFor(WMWidgetScreen(panel->alwChk), panel, wwin->wm_instance, wwin->wm_class, REVERT_TO_DEFAULT);
 
 	return panel;
 }
@@ -1361,6 +1365,7 @@ static void create_tab_window_advanced(WWindow *wwin, InspectorPanel *panel, int
 
 static void create_tab_icon_workspace(WWindow *wwin, InspectorPanel *panel)
 {
+	const char *db_icon;
 	WScreen *scr = wwin->screen_ptr;
 	int i = 0;
 
@@ -1390,7 +1395,8 @@ static void create_tab_icon_workspace(WWindow *wwin, InspectorPanel *panel)
 	panel->fileText = WMCreateTextField(panel->iconFrm);
 	WMMoveWidget(panel->fileText, 20, 105);
 	WMResizeWidget(panel->fileText, PWIDTH - (2 * 20) - (2 * 15), 20);
-	WMSetTextFieldText(panel->fileText, NULL);
+	db_icon = wDefaultGetIconFile(wwin->wm_instance, wwin->wm_class, False);
+	WMSetTextFieldText(panel->fileText, db_icon);
 	WMAddNotificationObserver(textEditedObserver, panel, WMTextDidEndEditingNotification, panel->fileText);
 
 	panel->alwChk = WMCreateSwitchButton(panel->iconFrm);
